import mergeWith from 'lodash.mergewith';
export { default as objectAssign } from 'object-assign';
export function omit(object, keys) {
  var result = {};
  Object.keys(object).forEach(key => {
    if (keys.includes(key)) return;
    result[key] = object[key];
  });
  return result;
}
export function pick(object, keys) {
  var result = {};
  keys.forEach(key => {
    if (key in object) {
      result[key] = object[key];
    }
  });
  return result;
}
export function split(object, keys) {
  var picked = {};
  var omitted = {};
  Object.keys(object).forEach(key => {
    if (keys.includes(key)) {
      picked[key] = object[key];
    } else {
      omitted[key] = object[key];
    }
  });
  return [picked, omitted];
}
/**
 * Get value from a deeply nested object using a string path.
 * Memoizes the value.
 * @param obj - the object
 * @param path - the string path
 * @param def  - the fallback value
 */

export function get(obj, path, fallback, index) {
  var key = typeof path === 'string' ? path.split('.') : [path];

  for (index = 0; index < key.length; index += 1) {
    if (!obj) {
      break;
    } // @ts-ignore


    obj = obj[key[index]];
  }

  return obj === undefined ? fallback : obj;
}

var memoize = fn => {
  var cache = new WeakMap();

  var memoizedFn = (obj, path, fallback, index) => {
    if (!cache.has(obj)) {
      cache.set(obj, new Map());
    }

    var map = cache.get(obj);
    var key = typeof path === 'string' ? path.split('.') : [path];

    if (map.has(key)) {
      return map.get(key);
    }

    var value = fn(obj, path, fallback, index);
    map.set(key, value);
    return value;
  };

  return memoizedFn;
};

export var memoizedGet = memoize(get);
/**
 * Get value from deeply nested object, based on path
 * It returns the path value if not found in object
 *
 * @param path - the string path or value
 * @param scale - the string path or value
 */

export function getWithDefault(path, scale) {
  return get(scale, path, path);
}

/**
 * Returns the items of an object that meet the condition specified in a callback function.
 *
 * @param object the object to loop through
 * @param fn The filter function
 */
export function objectFilter(object, fn) {
  var result = {};
  Object.keys(object).forEach(key => {
    var value = object[key];
    var shouldPass = fn(value, key, object);

    if (shouldPass) {
      result[key] = value;
    }
  });
  return result;
}
export var filterUndefined = object => objectFilter(object, val => val !== null && val !== undefined);
export var objectKeys = obj => Object.keys(obj);
/**
 * Object.entries polyfill for Nodev10 compatibility
 */

export var fromEntries = entries => entries.reduce((carry, _ref) => {
  var [key, value] = _ref;
  // @ts-ignore
  carry[key] = value;
  return carry;
}, {});
export { mergeWith };
//# sourceMappingURL=object.js.map